package com.logntw.eva.mybatis;

import java.io.Reader;
import java.util.List;

import org.apache.ibatis.io.Resources;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;

import com.logntw.eva.mybatis.entity.EntityTest;

public final class Main 
{
	private Main(){}
	public static void main(String... args) throws Exception
	{
        Reader reader = Resources.getResourceAsReader("mybatis-config.xml");  
         
        SqlSessionFactory sessionFactory = new SqlSessionFactoryBuilder().build(reader);  
          
        SqlSession session;                
        Mapper mapper;
        
        session = sessionFactory.openSession();                       
        mapper = session.getMapper(Mapper.class);
        mapper.createTable();
        session.close();
                    	
		long start, end;
		
		final int ENTITIES = 10000;	
		final int LOOP = 100;	
		final EntityTest[] ENTITY_ARR = new EntityTest[ENTITIES]; 
		

		start = System.currentTimeMillis();
		session = sessionFactory.openSession(ExecutorType.BATCH, false);        
	    mapper = session.getMapper(Mapper.class);		
		for(int i = 0; i < ENTITIES ; i ++)
		{			
			EntityTest entity = new EntityTest();
			entity.setId(i);
			entity.setName("ABC");
			entity.setAge(10);
			ENTITY_ARR[i] = entity;
			mapper.insert(entity);
		}
		session.commit();
		session.close();
		end = System.currentTimeMillis();
		System.out.println("The insertion of "+ENTITIES+" entities spent "+ (end - start) + " milliseconds.");
		
		
		
		
		
		start = System.currentTimeMillis();	
		session = sessionFactory.openSession(ExecutorType.BATCH, false);        
	    mapper = session.getMapper(Mapper.class);
	    for(int i = 0; i < LOOP ; i ++)
		{
			mapper.selectAll();
			session.clearCache(); // must be here.
		}		
		session.close();
		end = System.currentTimeMillis();
		System.out.println("The "+ LOOP + " times selection of the "+ENTITIES+" entities spent "+ (end - start) + " milliseconds.");
		
		
		
		
		start = System.currentTimeMillis();
		session = sessionFactory.openSession(ExecutorType.BATCH, false);        
	    mapper = session.getMapper(Mapper.class);
	    for(int i = 0; i < ENTITIES ; i ++)
		{
			EntityTest entity = ENTITY_ARR[i];
			entity.setName("DEF");
			mapper.updateName(entity);
		}
		session.commit();
		session.close();
		end = System.currentTimeMillis();
		System.out.println("The update of "+ENTITIES+" entities spent "+ (end - start) + " milliseconds.");
		
		
		
		
		start = System.currentTimeMillis();
		session = sessionFactory.openSession(ExecutorType.BATCH, false);        
	    mapper = session.getMapper(Mapper.class);
	    for(int i = 0; i < ENTITIES ; i ++)
		{
			EntityTest entity = ENTITY_ARR[i];
			mapper.delete(entity);
		}
		session.commit();
		session.close();
		end = System.currentTimeMillis();
		System.out.println("The deletion of "+ENTITIES+" entities spent "+ (end - start) + " milliseconds.");
		
		
		
		System.out.println("Press any key to exist.");
		System.in.read();
	}
}
